﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Takes the rest of the top item on the type stack.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorButLastExpression<T> : StatelessExpression {
    protected bool Eval(IPushStack<List<T>> vectorStack) {
      if (vectorStack.IsEmpty || vectorStack.Top.Count < 2)
        return false;

      var result = vectorStack.Top.GetRange(0, vectorStack.Top.Count - 1);
      vectorStack.SetTop(result);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].BUTLAST")]
  public class IntegerVectorButLastExpression : VectorButLastExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].BUTLAST")]
  public class FloatVectorButLastExpression : VectorButLastExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].BUTLAST")]
  public class BooleanVectorButLastExpression : VectorButLastExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "String[].BUTLAST")]
  public class StringVectorButLastExpression : VectorButLastExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack);
    }
  }
}
