﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Expressions {
  using System.Collections.Generic;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Attributes;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Stack;

  /// <summary>
  /// Pushes every item from the first vector onto the appropriate stack in reversed order.
  /// </summary>
  /// <typeparam name="T"></typeparam>
  public abstract class VectorPushAllExpression<T> : StatelessExpression {
    protected bool Eval(IPushStack<List<T>> vectorStack, IPushStack<T> literalStack) {
      if (vectorStack.IsEmpty)
        return false;

      var vector = vectorStack.Pop();
      vector.Reverse();
      literalStack.Push(vector);
      return true;
    }
  }

  [PushExpression(StackTypes.IntegerVector, "INTEGER[].PUSHALL", StackTypes.Integer)]
  public class IntegerVectorPushAllExpression : VectorPushAllExpression<long> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.IntegerVectorStack, interpreter.IntegerStack);
    }
  }

  [PushExpression(StackTypes.FloatVector, "FLOAT[].PUSHALL", StackTypes.Float)]
  public class FloatVectorPushAllExpression : VectorPushAllExpression<double> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.FloatVectorStack, interpreter.FloatStack);
    }
  }

  [PushExpression(StackTypes.BooleanVector, "BOOLEAN[].PUSHALL", StackTypes.Boolean)]
  public class BooleanVectorPushAllExpression : VectorPushAllExpression<bool> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.BooleanVectorStack, interpreter.BooleanStack);
    }
  }

  [PushExpression(StackTypes.StringVector, "STRING[].PUSHALL", StackTypes.String)]
  public class StringVectorPushAllExpression : VectorPushAllExpression<string> {
    public override bool Eval(IInternalPushInterpreter interpreter) {
      return Eval(interpreter.StringVectorStack, interpreter.StringStack);
    }
  }
}
